require 'koyori/kwic'

# String#reverse を日本語に対応するように再定義
class String
  def reverse
    self.split(//).reverse.join
  end
end

# Bilingual KWIC のクラス定義
class BilingualKWIC < Object
  def initialize(s_line,s_first,s_last,t_line,t_first,t_last,rank=KWIC_RANK_STANDARD)
    @s_line = s_line          # 原言語の1行
    @s_first = s_first        # 原言語中のキーワードの開始位置
    @s_last =  s_last         # 原言語中のキーワードの終了位置
    @t_line = t_line          # 対象言語の1行
    @t_first = t_first        # 対象言語中のキーワードの開始位置
    @t_last =  t_last         # 対象言語中のキーワードの終了位置
    @rank = rank              # ソートや表示の際に用いるランク
  end
  attr_reader :s_line, :s_first, :s_last, :t_line, :t_first, :t_last, :rank

  # KWIC のキーワードを返す．
  def s_key
    return @s_line[@s_first...@s_last]
  end
  def t_key
    return @t_line[@t_first...@t_last]
  end

  # 単一の言語に関して，KWIC形式で出力する
  def single_output(line,first,last,width,start_marker="",end_marker="")
    # line: 1行
    # first: キーワードの開始位置
    # last: キーワードの終了位置
    # width: テキストを表示する幅
    # start_marker,end_marker: キーワードの前後に置く記号

    len = last-first
    # 幅がキーワードの長さより狭いとき
    return start_marker + line[first,width] + end_marker if width < len

    left_margin = (width - len) / 2 / 2 * 2
    right_margin = width - len - left_margin

    if left_margin > first
      w = " " * (left_margin - first)
      left_edge = 0
    else
      w = ""
      left_edge = first - left_margin
    end

    w = w + line[left_edge...first] + start_marker + line[first...last] + end_marker
    right_edge = last + right_margin
    if right_edge > line.length
      w_end = " " * (right_edge - line.length)
      right_edge = line.length
    else
      w_end = ""
    end
    w = w + line[last...right_edge] + w_end     if line[last...right_edge]
  end

  # 両言語を並列に，KWIC形式で出力する
  def output(s_width=30,t_width=30,separator=" : ",start_marker="",end_marker="")
    s_output = single_output(@s_line,@s_first,@s_last,s_width,start_marker,end_marker)
    t_output = single_output(@t_line,@t_first,@t_last,t_width,start_marker,end_marker)
    if t_output
      s_output + separator + t_output
    else
      s_output + separator
    end
  end
end

# BilingualKWICの配列のためのクラス
class ArrayBilingualKWIC < Array
  def key_list
    # 対象言語の対応語を出現回数の多い順に出す
    # Dice係数順でなく，出現回数順なのはユーザは Dice係数を知らないから
    list = Hash.new(0)
    self.each{|k|
      list[k.t_key.downcase] += 1
    }
    list[""] = 0      # 対訳語の決定に失敗した物を最下位にする
    list.keys.sort{|x,y|  list[y] <=> list[x]}
  end

  def key_list_hash
    # 対応語のリストを，出現順序の値でハッシュ化したもの．sort で使用．

    list = self.key_list
    score = Hash.new(0)
    s = 1
    list.each{|x|
      score[x] = s
      s += 1
    }
    score
  end

  # 2種類の sort 
  # direction が false 以外なら，左方向へソート．
  # reverse を変更し，日本語にも一応対応
  # score の代わりに rank を使用するよう変更 (2005.3.22)
  def sort_by_source(direction=false)
    unless direction
      ary = self.sort_by{|l|
##	[score[l.t_line[l.t_first...l.t_last].downcase],l.s_line[l.s_first...-1].downcase]
	[l.rank,l.s_line[l.s_first...-1].downcase]
      }
    else
      ary = self.sort_by{|l|
	[l.rank,l.s_line[0...l.s_first].downcase.reverse]
      }
    end
    a = ArrayBilingualKWIC.new
    ary.each{|l| a << l}
    a
  end

  def sort_by_target(direction=false)
    if !direction
      ary = self.sort_by{|l|
#	[score[l.t_line[l.t_first...l.t_last].downcase],l.t_line[l.t_first...-1].downcase]
	[l.rank,l.t_line[l.t_first...-1].downcase]
      }
    else
      ary = self.sort_by{|l|
#	[score[l.t_line[l.t_first...l.t_last].downcase],l.t_line[0...l.t_first].downcase.reverse]
	[l.rank,l.t_line[0...l.t_first].downcase.reverse]
      }
    end
    a = ArrayBilingualKWIC.new
    ary.each{|l| a << l}
    a
  end

  # 通常の kwic を返す．
  def source_kwic
    a = ArrayKWIC.new
    self.each{|l|
      a << KWIC.new(l.s_line,l.s_first,l.s_last,nil,KWIC_RANK_STANDARD)
    }
    a
  end
  def target_kwic
    a = ArrayKWIC.new    
    self.each{|l|
      a << KWIC.new(l.t_line,l.t_first,l.t_last,nil,l.rank)
    }
    a
  end
end
